@@ -23,6 +23,7 @@
 #include <pwd.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <unistd.h>
 #include <string.h>
 #include <time.h>
 #include <unistd.h>
@@ -191,7 +192,7 @@ path_list_async_new (const char       *uri,
 
 	pli = path_list_data_new ();
 
-	escaped = gnome_vfs_escape_path_string (uri);
+	escaped = gnome_vfs_escape_host_and_path_string (uri);
 	if (escaped == NULL) {
 		path_list_data_free (pli);
 		if (f != NULL)
@@ -302,6 +303,71 @@ visit_rc_directory (const gchar *rc_dir,
 	return TRUE;
 }
 
+
+/* Checks all files in ~/RC_DIR and
+ */
+gboolean
+visit_rc_directory_sync (const gchar *rc_dir,
+			 const gchar *rc_ext,
+			 const gchar *dir,
+			 gboolean recursive,
+			 VisitFunc do_something,
+			 gpointer data)
+{
+	gchar *rc_dir_full_path;
+	GList *files, *dirs;
+	GList *scan;
+	gint   prefix_len, ext_len;
+	gchar *prefix;
+	
+	prefix = g_strconcat (g_get_home_dir(), 
+			      "/", 
+			      rc_dir,
+			      NULL);
+	prefix_len = strlen (prefix);
+	rc_dir_full_path = g_strconcat (prefix,
+					dir,
+					NULL);
+	g_free (prefix);
+	ext_len = strlen (rc_ext);
+
+	if (! path_is_dir (rc_dir_full_path)) {
+		g_free (rc_dir_full_path);
+		return FALSE;
+	}
+
+	path_list_new (rc_dir_full_path, &files, &dirs);
+
+	for (scan = files; scan; scan = scan->next) {
+		gchar *rc_file, *real_file;
+
+		rc_file = (gchar*) scan->data;
+		real_file = g_strndup (rc_file + prefix_len, 
+				       strlen (rc_file) - prefix_len - ext_len);
+		if (do_something)
+			(*do_something) (real_file, rc_file, data);
+		
+		g_free (real_file);
+	}
+
+	if (! recursive)
+		return TRUE;
+
+	for (scan = dirs; scan; scan = scan->next) {
+		gchar *sub_dir = (gchar*) scan->data;
+
+		visit_rc_directory_sync (rc_dir,
+					 rc_ext,
+					 sub_dir + prefix_len,
+					 TRUE, 
+					 do_something,
+					 data);
+	}
+
+	return TRUE;
+}
+
+
 /* -- browse_rc_directory_async implementation -- */
 
 typedef struct {
@@ -525,6 +591,49 @@ rmdir_recursive (const gchar *directory)
 /* -- */
 
 
+int
+check_permissions (const char *path,
+		   int         mode) 
+{
+	GnomeVFSFileInfo *info;
+	GnomeVFSResult    result;
+	char             *escaped;
+	int	          perm;
+	
+	info = gnome_vfs_file_info_new ();
+	escaped = gnome_vfs_escape_host_and_path_string (path);
+	result = gnome_vfs_get_file_info (escaped, 
+					  info, 
+					  (GNOME_VFS_FILE_INFO_DEFAULT 
+					   | GNOME_VFS_FILE_INFO_FOLLOW_LINKS));
+	
+	if (result != GNOME_VFS_OK) 
+		return FALSE; 
+
+	perm = 7;
+	
+	if ((mode & R_OK) && (info->permissions & 
+			      (GNOME_VFS_PERM_USER_READ | 
+			       GNOME_VFS_PERM_GROUP_READ|
+			       GNOME_VFS_PERM_OTHER_READ))) 
+		perm &= ~R_OK;
+
+	if ((mode & W_OK) && (info->permissions & 
+			      (GNOME_VFS_PERM_USER_WRITE | 
+			       GNOME_VFS_PERM_GROUP_WRITE|
+			       GNOME_VFS_PERM_OTHER_WRITE)))  
+		perm &= ~W_OK;
+	
+	if ((mode & X_OK) && (info->permissions & 
+			      (GNOME_VFS_PERM_USER_EXEC | 
+			       GNOME_VFS_PERM_GROUP_EXEC|
+			       GNOME_VFS_PERM_OTHER_EXEC)))  
+		perm &= ~X_OK;
+
+	return (perm);
+}
+
+
 gboolean
 path_is_file (const char *path)
 {
@@ -536,7 +645,8 @@ path_is_file (const char *path)
 	if (! path || ! *path) return FALSE; 
 
 	info = gnome_vfs_file_info_new ();
-	escaped = gnome_vfs_escape_path_string (path);
+	escaped = gnome_vfs_escape_host_and_path_string (path);
+
 	result = gnome_vfs_get_file_info (escaped, 
 					  info, 
 					  (GNOME_VFS_FILE_INFO_DEFAULT 
@@ -564,7 +674,8 @@ path_is_dir (const char *path)
 		return FALSE; 
 
 	info = gnome_vfs_file_info_new ();
-	escaped = gnome_vfs_escape_path_string (path);
+	escaped = gnome_vfs_escape_host_and_path_string (path);
+
 	result = gnome_vfs_get_file_info (escaped, 
 					  info, 
 					  (GNOME_VFS_FILE_INFO_DEFAULT 
@@ -572,8 +683,6 @@ path_is_dir (const char *path)
 	is_dir = FALSE;
 	if (result == GNOME_VFS_OK)
 		is_dir = (info->type == GNOME_VFS_FILE_TYPE_DIRECTORY);
-	else
-		g_warning ("%s: %s\n", path, gnome_vfs_result_to_string (result));
 
 	g_free (escaped);
 	gnome_vfs_file_info_unref (info);
@@ -617,7 +726,7 @@ get_file_size (const char *path)
 	if (! path || ! *path) return 0; 
 
 	info = gnome_vfs_file_info_new ();
-	escaped = gnome_vfs_escape_path_string (path);
+	escaped = gnome_vfs_escape_host_and_path_string (path);
 	result = gnome_vfs_get_file_info (escaped, 
 					  info,
 					  (GNOME_VFS_FILE_INFO_DEFAULT 
@@ -644,7 +753,7 @@ get_file_mtime (const char *path)
 	if (! path || ! *path) return 0; 
 
 	info = gnome_vfs_file_info_new ();
-	escaped = gnome_vfs_escape_path_string (path);
+	escaped = gnome_vfs_escape_host_and_path_string (path);
 	result = gnome_vfs_get_file_info (escaped, 
 					  info, 
 					  (GNOME_VFS_FILE_INFO_DEFAULT 
@@ -672,7 +781,7 @@ get_file_ctime (const gchar *path)
 	if (! path || ! *path) return 0; 
 
 	info = gnome_vfs_file_info_new ();
-	escaped = gnome_vfs_escape_path_string (path);
+	escaped = gnome_vfs_escape_host_and_path_string (path);
 	result = gnome_vfs_get_file_info (escaped, 
 					  info, 
 					  (GNOME_VFS_FILE_INFO_DEFAULT 
@@ -700,7 +809,7 @@ set_file_mtime (const gchar *path,
 	file_info->mtime = mtime;
 	file_info->atime = mtime;
 
-	escaped_path = gnome_vfs_escape_path_string (path);
+	escaped_path = gnome_vfs_escape_host_and_path_string (path);
 	gnome_vfs_set_file_info (escaped_path,
 				 file_info,
 				 GNOME_VFS_SET_FILE_INFO_TIME);
@@ -713,10 +822,10 @@ gboolean
 file_copy (const char *from, 
 	   const char *to)
 {
-	FILE *fin, *fout;
-	char  buf[BUF_SIZE];
-	char *dest_dir;
-	int   n;
+	FILE   *fin, *fout;
+	char    buf[BUF_SIZE];
+	char   *dest_dir;
+	size_t  n;
 
 	if (strcmp (from, to) == 0) {
 		g_warning ("cannot copy file %s: source and destination are the same\n", from);
@@ -754,7 +863,7 @@ file_copy (const char *from,
 		}
 	}
 	
-	if ((n < 0) && (errno == EINTR))
+	if (errno == EINTR)
 		goto retry_read;
 
 	g_free (dest_dir);
@@ -881,9 +990,15 @@ file_is_image (const gchar *name,
 	const char *result;
 	gboolean    is_an_image;
 
-	if (fast_file_type)
-		result = gnome_vfs_mime_type_from_name_or_default (name, NULL);
-	else 
+	if (fast_file_type) {
+		char *n1 = g_filename_to_utf8 (name, -1, 0, 0, 0);
+		char *n2 = g_utf8_strdown (n1, -1);
+		char *n3 = g_filename_from_utf8 (n2, -1, 0, 0, 0);
+		result = gnome_vfs_mime_type_from_name_or_default (n3, NULL);
+		g_free (n3);
+		g_free (n2);
+		g_free (n1);
+	} else 
 		result = gnome_vfs_get_file_mime_type (name, NULL, FALSE);
 
 	/* Unknown file type. */
@@ -903,9 +1018,15 @@ image_is_jpeg (const char *name)
 {
 	const char *result;
 
-	if (eel_gconf_get_boolean (PREF_FAST_FILE_TYPE, TRUE))
-		result = gnome_vfs_mime_type_from_name_or_default (name, NULL);
-	else 
+	if (eel_gconf_get_boolean (PREF_FAST_FILE_TYPE, TRUE)) {
+		char *n1 = g_filename_to_utf8 (name, -1, 0, 0, 0);
+		char *n2 = g_utf8_strdown (n1, -1);
+		char *n3 = g_filename_from_utf8 (n2, -1, 0, 0, 0);
+		result = gnome_vfs_mime_type_from_name_or_default (n3, NULL);
+		g_free (n3);
+		g_free (n2);
+		g_free (n1);
+	} else 
 		result = gnome_vfs_get_file_mime_type (name, NULL, FALSE);
 	
 	/* Unknown file type. */
@@ -1035,6 +1156,25 @@ remove_ending_separator (const gchar *path)
 }
 
 
+/* Check whether the path_src is contained in path_dest */
+gboolean
+path_in_path (const char  *path_src,
+	      const char  *path_dest)
+{
+	int path_src_l, path_dest_l;
+
+	if ((path_src == NULL) || (path_dest == NULL))
+		return FALSE;
+	
+	path_src_l = strlen (path_src);
+	path_dest_l = strlen (path_dest);
+	
+	return ((path_dest_l > path_src_l)
+		&& (strncmp (path_src, path_dest, path_src_l) == 0)
+		&& (path_dest[path_src_l] == '/'));
+}
+
+
 gboolean 
 ensure_dir_exists (const char *a_path, 
 		   mode_t      mode)
@@ -1048,8 +1188,13 @@ ensure_dir_exists (const char *a_path,
 		return TRUE;
 
 	path = g_strdup (a_path);
-	p = path;
-	
+
+	p = strstr (path, "://");
+	if (p == NULL)   /* Not a URI */
+		p = path;
+	else  /* Is a URI */
+		p = p + 3;  /* Move p past the :// */
+
 	while (*p != '\0') {
 		p++;
 		if ((*p == '/') || (*p == '\0')) {
@@ -1061,7 +1206,8 @@ ensure_dir_exists (const char *a_path,
 			}
 			
 			if (! path_is_dir (path)) {
-				if (mkdir (path, mode) < 0) {
+				GnomeVFSResult result = gnome_vfs_make_directory (path, mode);
+				if (result != GNOME_VFS_OK) {
 					g_warning ("directory creation failed: %s.", path);
 					g_free (path);
 					return FALSE;
@@ -1089,7 +1235,7 @@ dir_list_filter_and_sort (GList *dir_list,
 	filtered = NULL;
 	scan = dir_list;
 	while (scan) {
-		const gchar *name_only = file_name_from_path (scan->data);
+		const char *name_only = file_name_from_path (scan->data);
 
 		if (! (file_is_hidden (name_only) && ! show_dot_files) 
 		    && (strcmp (name_only, CACHE_DIR) != 0)) {
@@ -1402,10 +1548,18 @@ GnomeVFSURI *
 new_uri_from_path (const char *path)
 {
 	char        *escaped;
+	char        *uri_txt;
 	GnomeVFSURI *uri;
 
-	escaped = gnome_vfs_escape_path_string (path);
-	uri = gnome_vfs_uri_new (escaped);
+	escaped = gnome_vfs_escape_host_and_path_string (path);
+	if (escaped[0] == '/')
+		uri_txt = g_strconcat ("file://", escaped, NULL);
+	else
+		uri_txt = g_strdup (escaped);
+
+	uri = gnome_vfs_uri_new (uri_txt);
+
+	g_free (uri_txt);
 	g_free (escaped);
 
 	g_return_val_if_fail (uri != NULL, NULL);
