@@ -3,7 +3,7 @@
 /*
  *  GThumb
  *
- *  Copyright (C) 2001 The Free Software Foundation, Inc.
+ *  Copyright (C) 2001, 2003 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 #include <config.h>
 #include <libgnome/libgnome.h>
 #include <gtk/gtk.h>
+#include "gconf-utils.h"
 
 
 #define REQUEST_ENTRY_WIDTH 220
@@ -106,6 +107,7 @@ _gtk_message_dialog_new (GtkWindow        *parent,
 			 GtkDialogFlags    flags,
 			 const char       *stock_id,
 			 const char       *message,
+			 const char       *secondary_message,
 			 const char       *first_button_text,
 			 ...)
 {
@@ -116,18 +118,14 @@ _gtk_message_dialog_new (GtkWindow        *parent,
 	va_list       args;
 	const gchar  *text;
 	int           response_id;
-	GtkStockItem  item;
-	char         *title;
+	char         *escaped_message, *markup_text;
+
+	g_return_val_if_fail (message != NULL, NULL);
 
 	if (stock_id == NULL)
 		stock_id = GTK_STOCK_DIALOG_INFO;
 
-	if (gtk_stock_lookup (stock_id, &item))
-		title = item.label;
-	else
-		title = _("gThumb");
-
-	d = gtk_dialog_new_with_buttons (title, parent, flags, NULL);
+	d = gtk_dialog_new_with_buttons ("", parent, flags, NULL);
 	gtk_window_set_resizable (GTK_WINDOW (d), FALSE);
 
 	gtk_dialog_set_has_separator (GTK_DIALOG (d), FALSE);
@@ -140,7 +138,21 @@ _gtk_message_dialog_new (GtkWindow        *parent,
 	image = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_DIALOG);
 	gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0.0);
 
-	label = gtk_label_new (message);	
+	label = gtk_label_new ("");
+
+	escaped_message = g_markup_escape_text (message, -1);
+	if (secondary_message != NULL) {
+		char *escaped_secondary_message = g_markup_escape_text (secondary_message, -1);
+		markup_text = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s", 
+					       escaped_message,
+					       escaped_secondary_message);
+		g_free (escaped_secondary_message);
+	} else 
+		markup_text = g_strdup (escaped_message);
+	gtk_label_set_markup (GTK_LABEL (label), markup_text);
+	g_free (markup_text);
+	g_free (escaped_message);
+
 	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
 	gtk_label_set_selectable (GTK_LABEL (label), TRUE);
 	
@@ -202,18 +214,10 @@ _gtk_request_dialog_run (GtkWindow        *parent,
 	GtkWidget    *vbox;
 	GtkWidget    *entry;
 	GtkWidget    *button;
-	GtkStockItem  item;
-	char         *title;
-	char         *stock_id;
 	char         *result = NULL;
+	char         *stock_id = GTK_STOCK_DIALOG_QUESTION;
 
-	stock_id = GTK_STOCK_DIALOG_QUESTION;
-	if (gtk_stock_lookup (stock_id, &item))
-		title = item.label;
-	else
-		title = _("gThumb");
-
-	d = gtk_dialog_new_with_buttons (title, parent, flags, NULL);
+	d = gtk_dialog_new_with_buttons ("", parent, flags, NULL);
 	gtk_window_set_resizable (GTK_WINDOW (d), FALSE);
 
 	gtk_dialog_set_has_separator (GTK_DIALOG (d), FALSE);
@@ -284,11 +288,9 @@ _gtk_request_dialog_run (GtkWindow        *parent,
 
 	/* Run dialog */
 
-	if (gtk_dialog_run (GTK_DIALOG (d)) == GTK_RESPONSE_YES) {
-		const char *text;
-		text = gtk_entry_get_text (GTK_ENTRY (entry));
-		result = g_locale_from_utf8 (text, -1, NULL, NULL, NULL);
-	} else
+	if (gtk_dialog_run (GTK_DIALOG (d)) == GTK_RESPONSE_YES) 
+		result = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
+	else
 		result = NULL;
 
 	gtk_widget_destroy (d);
@@ -309,16 +311,9 @@ _gtk_yesno_dialog_new (GtkWindow        *parent,
 	GtkWidget    *image;
 	GtkWidget    *hbox;
 	GtkWidget    *button;
-	GtkStockItem  item;
-	char         *title;
 	char         *stock_id = GTK_STOCK_DIALOG_QUESTION;
 
-	if (gtk_stock_lookup (stock_id, &item))
-		title = item.label;
-	else
-		title = _("gThumb");
-
-	d = gtk_dialog_new_with_buttons (title, parent, flags, NULL);
+	d = gtk_dialog_new_with_buttons ("", parent, flags, NULL);
 	gtk_window_set_resizable (GTK_WINDOW (d), FALSE);
 
 	gtk_dialog_set_has_separator (GTK_DIALOG (d), FALSE);
@@ -372,6 +367,99 @@ _gtk_yesno_dialog_new (GtkWindow        *parent,
 }
 
 
+static void
+yesno__check_button_toggled_cb (GtkToggleButton *button,
+				const char      *gconf_key)
+{
+	eel_gconf_set_boolean (gconf_key, 
+			       ! gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)));
+}
+
+
+GtkWidget*
+_gtk_yesno_dialog_with_checkbutton_new (GtkWindow        *parent,
+					GtkDialogFlags    flags,
+					const char       *message,
+					const char       *no_button_text,
+					const char       *yes_button_text,
+					const char       *check_button_label,
+					const char       *gconf_key)
+{
+	GtkWidget *d;
+	GtkWidget *label;
+	GtkWidget *image;
+	GtkWidget *hbox;
+	GtkWidget *button;
+	GtkWidget *check_button;
+	char      *stock_id = GTK_STOCK_DIALOG_QUESTION;
+
+	d = gtk_dialog_new_with_buttons ("", parent, flags, NULL);
+	gtk_window_set_resizable (GTK_WINDOW (d), FALSE);
+
+	gtk_dialog_set_has_separator (GTK_DIALOG (d), FALSE);
+	gtk_container_set_border_width (GTK_CONTAINER (d), 6);
+	gtk_container_set_border_width (GTK_CONTAINER (GTK_DIALOG (d)->vbox), 6);
+	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (d)->vbox), 8);
+
+	/* Add label and image */
+
+	image = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_DIALOG);
+	gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0.0);
+
+	label = gtk_label_new (message);	
+	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+	gtk_label_set_selectable (GTK_LABEL (label), TRUE);
+	
+	hbox = gtk_hbox_new (FALSE, 12);
+	gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
+	
+	gtk_box_pack_start (GTK_BOX (hbox), image,
+			    FALSE, FALSE, 0);
+	
+	gtk_box_pack_start (GTK_BOX (hbox), label,
+			    TRUE, TRUE, 0);
+	
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (d)->vbox),
+			    hbox,
+			    FALSE, FALSE, 0);
+
+	/* Add checkbutton */
+
+	check_button = gtk_check_button_new_with_mnemonic (check_button_label);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (d)->vbox),
+			    check_button,
+			    FALSE, FALSE, 0);
+	gtk_widget_show (check_button);
+	
+	gtk_widget_show_all (hbox);
+
+	/* Add buttons */
+
+	button = create_button (GTK_STOCK_CANCEL, no_button_text);
+	gtk_dialog_add_action_widget (GTK_DIALOG (d), 
+				      button, 
+				      GTK_RESPONSE_CANCEL);
+
+	/**/
+
+	button = create_button (GTK_STOCK_OK, yes_button_text);
+	gtk_dialog_add_action_widget (GTK_DIALOG (d), 
+				      button, 
+				      GTK_RESPONSE_YES);
+
+	/**/
+
+	gtk_dialog_set_default_response (GTK_DIALOG (d), GTK_RESPONSE_YES);
+
+	g_signal_connect (G_OBJECT (check_button),
+			  "toggled", 
+			  G_CALLBACK (yesno__check_button_toggled_cb), 
+			  (gpointer) gconf_key);
+	
+	return d;
+}
+
+
 void
 _gtk_error_dialog_from_gerror_run (GtkWindow        *parent,
 				   GError          **gerror)
@@ -379,19 +467,21 @@ _gtk_error_dialog_from_gerror_run (GtkWindow        *parent,
 	GtkWidget *d;
 
 	g_return_if_fail (*gerror != NULL);
+	g_return_if_fail ((*gerror)->message != NULL);
 
-	d = _gtk_message_dialog_new (parent,
-				     GTK_DIALOG_MODAL,
-				     GTK_STOCK_DIALOG_ERROR,
-				     (*gerror)->message,
-				     GTK_STOCK_OK, GTK_RESPONSE_CANCEL,
-				     NULL);
-
-	g_signal_connect (G_OBJECT (d), "response",
-			  G_CALLBACK (gtk_widget_destroy),
-			  NULL);
-
-	gtk_widget_show (d);
+	if ((*gerror)->message != NULL) {
+		d = _gtk_message_dialog_new (parent,
+					     GTK_DIALOG_MODAL,
+					     GTK_STOCK_DIALOG_ERROR,
+					     (*gerror)->message,
+					     NULL,
+					     GTK_STOCK_OK, GTK_RESPONSE_CANCEL,
+					     NULL);
+		g_signal_connect (G_OBJECT (d), "response",
+				  G_CALLBACK (gtk_widget_destroy),
+				  NULL);
+		gtk_widget_show (d);
+	}
 
 	g_clear_error (gerror);
 }
@@ -414,6 +504,7 @@ _gtk_error_dialog_run (GtkWindow        *parent,
 				      GTK_DIALOG_MODAL,
 				      GTK_STOCK_DIALOG_ERROR,
 				      message,
+				      NULL,
 				      GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL,
 				      NULL);
 	g_free (message);
@@ -443,6 +534,7 @@ _gtk_info_dialog_run (GtkWindow        *parent,
 				      GTK_DIALOG_MODAL,
 				      GTK_STOCK_DIALOG_INFO,
 				      message,
+				      NULL,
 				      GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL,
 				      NULL);
 	g_free (message);
@@ -461,6 +553,9 @@ _gtk_entry_set_locale_text (GtkEntry   *entry,
 {
 	char *utf8_text;
 
+	if (text == NULL)
+		text = "";
+
 	utf8_text = g_locale_to_utf8 (text, -1, NULL, NULL, NULL);
 	gtk_entry_set_text (entry, utf8_text);
 	g_free (utf8_text);
@@ -489,6 +584,9 @@ _gtk_label_set_locale_text (GtkLabel   *label,
 {
 	char *utf8_text;
 
+	if (text == NULL)
+		text = "";
+
 	utf8_text = g_locale_to_utf8 (text, -1, NULL, NULL, NULL);
 	gtk_label_set_text (label, utf8_text);
 	g_free (utf8_text);
@@ -510,3 +608,64 @@ _gtk_label_get_locale_text (GtkLabel *label)
 	return text;
 }
 
+
+void
+_gtk_entry_set_filename_text (GtkEntry   *entry,
+			      const char *text)
+{
+	char *utf8_text;
+
+	if (text == NULL)
+		text = "";
+
+	utf8_text = g_filename_to_utf8 (text, -1, NULL, NULL, NULL);
+	gtk_entry_set_text (entry, utf8_text);
+	g_free (utf8_text);
+}
+
+
+char *
+_gtk_entry_get_filename_text (GtkEntry   *entry)
+{
+	const char *utf8_text;
+	char       *text;
+
+	utf8_text = gtk_entry_get_text (entry);
+	if (utf8_text == NULL)
+		return NULL;
+
+	text = g_filename_from_utf8 (utf8_text, -1, NULL, NULL, NULL);
+
+	return text;
+}
+
+
+void
+_gtk_label_set_filename_text (GtkLabel   *label,
+			      const char *text)
+{
+	char *utf8_text;
+
+	if (text == NULL)
+		text = "";
+
+	utf8_text = g_filename_to_utf8 (text, -1, NULL, NULL, NULL);
+	gtk_label_set_text (label, utf8_text);
+	g_free (utf8_text);
+}
+
+
+char *
+_gtk_label_get_filename_text (GtkLabel   *label)
+{
+	const char *utf8_text;
+	char       *text;
+
+	utf8_text = gtk_label_get_text (label);
+	if (utf8_text == NULL)
+		return NULL;
+
+	text = g_filename_from_utf8 (utf8_text, -1, NULL, NULL, NULL);
+
+	return text;
+}
