@@ -1443,7 +1443,6 @@ copy_next_file (FileCopyData *fcdata)
 		char *dest_file;
 		char *src_cache_file;
 		char *dest_cache_file;
-		char *src_cache_dir;
 
 		dest_file = g_strconcat (fcdata->destination,
 					 "/", 
@@ -1451,15 +1450,8 @@ copy_next_file (FileCopyData *fcdata)
 					 NULL);
 		
 		src_cache_file = comments_get_comment_filename (src_file, TRUE, TRUE);
-		if (! path_is_file (src_cache_file)) {
-			g_free (src_cache_file);
-			src_cache_file = comments_get_comment_filename (src_file, FALSE, TRUE);
-		}
-
 		dest_cache_file = comments_get_comment_filename (dest_file, TRUE, TRUE);
 
-		src_cache_dir = remove_level_from_path (src_cache_file);
-
 		if (path_is_file (src_cache_file)) {
 			char *parent_dir;
 
@@ -1471,7 +1463,6 @@ copy_next_file (FileCopyData *fcdata)
 			dest_list = g_list_append (dest_list, new_uri_from_path (dest_cache_file));
 		}
 
-		g_free (src_cache_dir);
 		g_free (src_cache_file);
 		g_free (dest_cache_file);
 
@@ -1480,8 +1471,6 @@ copy_next_file (FileCopyData *fcdata)
 		src_cache_file = cache_get_nautilus_cache_name (src_file);
 		dest_cache_file = cache_get_nautilus_cache_name (dest_file);
 
-		src_cache_dir = remove_level_from_path (src_cache_file);
-		
 		if (path_is_file (src_cache_file)) {
 			char *parent_dir = remove_level_from_path (dest_cache_file);
 			ensure_dir_exists (parent_dir, 0755);
@@ -1491,7 +1480,6 @@ copy_next_file (FileCopyData *fcdata)
 			dest_list = g_list_append (dest_list, new_uri_from_path (dest_cache_file));
 		}
 		
-		g_free (src_cache_dir);
 		g_free (src_cache_file);
 		g_free (dest_cache_file);
 
@@ -1650,7 +1638,7 @@ dlg_files_move_to_trash (GThumbWindow   *window,
 
 	g_return_if_fail (file_list != NULL);
 
-	e_path = gnome_vfs_escape_path_string (file_list->data);
+	e_path = gnome_vfs_escape_host_and_path_string (file_list->data);
 	first_uri = gnome_vfs_uri_new (e_path);
 	g_free (e_path);
 	
@@ -2058,7 +2046,7 @@ static int
 folder_progress_sync_cb (GnomeVFSXferProgressInfo *info,
 			 gpointer                  data)
 {
-	int ret_val;
+	int ret_val = TRUE;
 
 	if (info->status == GNOME_VFS_XFER_PROGRESS_STATUS_OK) {
 		ret_val = TRUE;
@@ -2137,7 +2125,7 @@ folder_copy (GThumbWindow   *window,
 	if (file_op != FILE_OP_DELETE) {
 		if ((src_path == NULL) || (dest_path == NULL))
 			return;
-		if (strncmp (src_path, dest_path, strlen (src_path)) == 0)
+		if (path_in_path (src_path, dest_path))
 			return;
 	}
 
@@ -2197,48 +2185,19 @@ folder_copy (GThumbWindow   *window,
 		dest_list = g_list_append (dest_list, new_uri_from_path (dest_path));
 
 	if (fcdata->include_cache) {
-		char *src_cache;
-		char *dest_cache;
 		char *src_folder_comment;
 
-		src_cache = comments_get_comment_dir (src_path, TRUE, TRUE);
-		if (! path_is_dir (src_cache)) {
-			g_free (src_cache);
-			src_cache = comments_get_comment_dir (src_path, FALSE, TRUE);
-		}
-
-		dest_cache = comments_get_comment_dir (dest_path, TRUE, TRUE);
-
-		if (path_is_dir (src_cache)) {
-			char *parent_dir = remove_level_from_path (dest_cache);
-			ensure_dir_exists (parent_dir, 0755);
-			g_free (parent_dir);
-
-			src_list = g_list_append (src_list, 
-						  new_uri_from_path (src_cache));
-
-			if (fcdata->file_op != FILE_OP_DELETE) {
-				if (path_is_dir (dest_cache) && !overwrite_all)
-					rmdir_recursive (dest_cache);
-
-				dest_list = g_list_append (dest_list, 
-							   new_uri_from_path (dest_cache));
-			}
-		}
-
-		g_free (src_cache);
-		g_free (dest_cache);
-
-		/**/
-
 		src_folder_comment = comments_get_comment_filename (src_path, TRUE, TRUE);
 
 		if (path_is_file (src_folder_comment)) {
 			char *dest_folder_comment;
 			dest_folder_comment = comments_get_comment_filename (dest_path, TRUE, TRUE);
 			src_list = g_list_append (src_list, new_uri_from_path (src_folder_comment));
-			if (fcdata->file_op != FILE_OP_DELETE)
+			if (fcdata->file_op != FILE_OP_DELETE) {
+				if (path_is_file (dest_folder_comment))
+					unlink (dest_folder_comment);
 				dest_list = g_list_append (dest_list, new_uri_from_path (dest_folder_comment));
+			}
 			g_free (dest_folder_comment);
 		}
 
@@ -2317,7 +2276,7 @@ dlg_folder_move_to_trash (GThumbWindow   *window,
 	GnomeVFSURI     *parent_uri, *trash_uri;
 
 	parent_dir = remove_level_from_path (folder);
-	e_parent_dir = gnome_vfs_escape_path_string (parent_dir);
+	e_parent_dir = gnome_vfs_escape_host_and_path_string (parent_dir);
 	parent_uri = gnome_vfs_uri_new (e_parent_dir);
 	
 	gnome_vfs_find_directory (parent_uri, 
@@ -2385,6 +2344,7 @@ typedef struct {
 	char           *destination;
 	gboolean        remove_source;
 	gboolean        include_cache;
+	gboolean        overwrite_all;
 	FileOpDoneFunc  done_func;
 	gpointer        done_data;
 } CopyItemsData;
@@ -2506,8 +2466,8 @@ copy_current_item (CopyItemsData *cidata)
 					cidata->file_list,
 					cidata->destination,
 					cidata->remove_source,
-					TRUE,
-					FALSE,
+					cidata->include_cache,
+					cidata->overwrite_all,
 					copy_item__continue2,
 					cidata);
 
@@ -2526,7 +2486,7 @@ copy_current_item (CopyItemsData *cidata)
 				 dest_path,
 				 cidata->remove_source,
 				 cidata->include_cache,
-				 FALSE,
+				 cidata->overwrite_all,
 				 copy_item__continue1,
 				 cidata);
 
@@ -2541,6 +2501,7 @@ dlg_copy_items (GThumbWindow   *window,
 		const char     *destination,
 		gboolean        remove_source,
 		gboolean        include_cache,
+		gboolean        overwrite_all,
 		FileOpDoneFunc  done_func,
 		gpointer        done_data)
 {
@@ -2560,6 +2521,7 @@ dlg_copy_items (GThumbWindow   *window,
 	cidata->destination = g_strdup (destination);
 	cidata->remove_source = remove_source;
 	cidata->include_cache = include_cache;
+	cidata->overwrite_all = overwrite_all;
 
 	for (scan = cidata->item_list; scan; scan = scan->next) {
 		char *path = scan->data;
