@@ -139,7 +139,6 @@ typedef struct {
 	GList          *dirs;
 
 	char           *catalog_path;
-	gboolean        search_comments;
 
 	GHashTable     *folders_comment;
 } DialogData;
@@ -255,7 +254,7 @@ search_clicked_cb (GtkWidget  *widget,
 	entry = gtk_entry_get_text (GTK_ENTRY (data->s_filename_entry));
 	search_data_set_file_pattern (data->search_data, entry);
 	if (entry != NULL) 
-		data->file_patterns = search_util_get_patterns (entry);
+		data->file_patterns = search_util_get_file_patterns (entry);
 
 	/* * comment pattern */
 
@@ -522,6 +521,7 @@ dlg_search_ui (GThumbWindow *window,
 	DialogData        *data;
 	GtkCellRenderer   *renderer;
 	GtkTreeViewColumn *column;
+	GValue             value = {0, };
 
 	data = g_new0 (DialogData, 1);
 
@@ -587,6 +587,14 @@ dlg_search_ui (GThumbWindow *window,
 
 	/* Set widgets data. */
 
+	/* Make use of the new filechooser */
+
+	g_value_init (&value, G_TYPE_BOOLEAN);
+	g_value_set_boolean (&value, TRUE);
+	g_object_set_property (G_OBJECT (data->s_start_from_fileentry),"use_filechooser", &value);
+	
+	/**/
+
 	if (catalog_path == NULL) {
 		if (data->window->dir_list->path != NULL)
 			_gtk_entry_set_filename_text (GTK_ENTRY (data->s_start_from_entry), data->window->dir_list->path);
@@ -751,7 +759,7 @@ match_patterns (char       **patterns,
 	for (i = 0; patterns[i] != NULL; i++) 
 		if (g_utf8_fnmatch (patterns[i], string, FNM_CASEFOLD) == 0)
 			return TRUE;
-
+	
 	return FALSE;
 }
 
@@ -889,7 +897,7 @@ file_respects_search_criteria (DialogData *data,
 		match_date = TRUE;
 
 	home_dir = g_get_home_dir ();
-	if (strncmp (home_dir, filename, strlen (home_dir)) == 0)
+	if (path_in_path (home_dir, filename))
 		name_only = filename + strlen (home_dir);
 	else
 		name_only = filename;
@@ -946,43 +954,6 @@ add_file_list (DialogData *data, GList *file_list)
 }
 
 
-static char *
-uri_from_comment_uri (const char *comment_uri,
-		      gboolean    is_file)
-{
-	char *base;
-	char *uri;
-	int   comment_uri_l;
-	int   base_l;
-
-	if (comment_uri == NULL)
-		return NULL;
-
-	comment_uri_l = strlen (comment_uri);
-
-	base = comments_get_comment_dir (NULL, TRUE, TRUE);
-	base_l = strlen (base);
-
-	if (comment_uri_l == base_l) 
-		uri = g_strdup ("/");
-
-	else if (comment_uri_l > base_l) {
-		uri = g_strdup (comment_uri + base_l);
-
-		if (is_file) { /* remove the comment extension. */
-			int uri_l = comment_uri_l - base_l;
-			uri [uri_l - strlen (COMMENT_EXT)] = 0;
-		}
-
-	} else
-		uri = NULL;
-	
-	g_free (base);
-
-	return uri;
-}
-
-
 static void search_dir_async (DialogData *data, gchar *dir);
 
 
@@ -1028,13 +999,6 @@ directory_load_cb (GnomeVFSAsyncHandle *handle,
 			str_uri = gnome_vfs_uri_to_string (full_uri, GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD);
 			unesc_uri = gnome_vfs_unescape_string (str_uri, NULL);
 
-			if (data->search_comments) {
-				char *tmp;
-				tmp = uri_from_comment_uri (unesc_uri, TRUE);
-				g_free (unesc_uri);
-				unesc_uri = tmp;
-			}
-
 			if (file_respects_search_criteria (data, unesc_uri)) 
 				files = g_list_prepend (files, unesc_uri);
 			else
@@ -1051,13 +1015,6 @@ directory_load_cb (GnomeVFSAsyncHandle *handle,
 			str_uri = gnome_vfs_uri_to_string (full_uri, GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD);
 			unesc_uri = gnome_vfs_unescape_string (str_uri, NULL);
 
-			if (data->search_comments) {
-				char *tmp;
-				tmp = uri_from_comment_uri (unesc_uri, FALSE);
-				g_free (unesc_uri);
-				unesc_uri = tmp;
-			}
-
 			data->dirs = g_list_prepend (data->dirs, unesc_uri);
 			g_free (str_uri);
 			break;
@@ -1121,10 +1078,7 @@ search_dir_async (DialogData *data, char *dir)
 	char *escaped;
 	char *start_from;
 
-	if (data->search_comments) 
-		start_from = comments_get_comment_dir (dir, TRUE, TRUE);
-	else
-		start_from = g_strdup (dir);
+	start_from = g_strdup (dir);
 
 	_gtk_entry_set_filename_text (GTK_ENTRY (data->p_current_dir_entry), dir);
 
@@ -1133,7 +1087,7 @@ search_dir_async (DialogData *data, char *dir)
 	if (data->uri != NULL)
 		gnome_vfs_uri_unref (data->uri);
 
-	escaped = gnome_vfs_escape_path_string (start_from);
+	escaped = gnome_vfs_escape_host_and_path_string (start_from);
 	data->uri = gnome_vfs_uri_new (escaped);
 
 	g_free (start_from);
@@ -1167,17 +1121,6 @@ search_images_async (DialogData *data)
 	free_search_results_data (data);
 	gtk_list_store_clear (data->p_progress_tree_model);
 
-	/* if the search criteria include comment data just search in
-	 * the comments tree */
-
-	/* FIXME 
-	data->search_comments = ! (empty_pattern (search_data->comment_pattern)
-				   && empty_pattern (search_data->place_pattern)
-				   && empty_pattern (search_data->keywords_pattern));
-	*/
-
-	data->search_comments = FALSE;
-
 	search_dir_async (data, search_data->start_from);
 }
 
