@@ -3,7 +3,7 @@
 /*
  *  GThumb
  *
- *  Copyright (C) 2001, 2003 Free Software Foundation, Inc.
+ *  Copyright (C) 2001, 2003, 2004 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -27,8 +27,8 @@
 #include <gdk/gdkkeysyms.h>
 #include <gdk/gdkx.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
+#include "actions.h"
 #include "comments.h"
-#include "commands-impl.h"
 #include "fullscreen.h"
 #include "glib-utils.h"
 #include "gconf-utils.h"
@@ -36,6 +36,7 @@
 #include "main.h"
 #include "pixbuf-utils.h"
 #include "gth-exif-utils.h"
+#include "preferences.h"
 
 #include "icons/pixbufs.h"
 
@@ -54,17 +55,16 @@ static FullScreen     *current_fullscreen;
 
 
 static void
-set_command_state (GThumbWindow *window,
-		   char         *cname,
-		   gboolean      setted)
+set_action_active (GThumbWindow *window,
+		   char         *action_name,
+		   gboolean      is_active)
 {
-	char *full_cname = g_strconcat ("/commands/", cname, NULL);
-	char *new_value  = setted ? "1" : "0";
-	bonobo_ui_component_set_prop (window->ui_component, 
-				      full_cname, 
-				      "state", new_value,
-				      NULL);
-	g_free (full_cname);
+	GtkAction *action;
+	action = gtk_action_group_get_action (window->actions, action_name);
+	if (action == NULL) 
+		g_print ("%s NON ESISTE\n", action_name);
+	else
+		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), is_active);
 }
 
 
@@ -500,6 +500,16 @@ image_key_press_cb (GtkWidget   *widget,
 		image_viewer_set_zoom (viewer, 1.0);
 		break;
 
+		/* Set zoom to 2.0. */
+	case GDK_2:
+		image_viewer_set_zoom (viewer, 2.0);
+		break;
+
+		/* Set zoom to 3.0. */
+	case GDK_3:
+		image_viewer_set_zoom (viewer, 3.0);
+		break;
+
 		/* Zoom to fit. */
 	case GDK_x:
 		image_viewer_zoom_to_fit (viewer);
@@ -515,12 +525,12 @@ image_key_press_cb (GtkWidget   *widget,
 
 		/* Toggle animation. */
 	case GDK_g:
-		set_command_state (window, "View_PlayAnimation", ! viewer->play_animation);
+		set_action_active (window, "View_PlayAnimation", ! viewer->play_animation);
 		break;
 
 		/* Step animation. */
 	case GDK_j:
-		view_step_ani_command_impl (NULL, window, NULL);
+		activate_action_view_step_animation (NULL, window);
 		break;
 
 		/* Delete selection. */
@@ -538,9 +548,9 @@ image_key_press_cb (GtkWidget   *widget,
 		/* Delete. */
 
 		if (window->sidebar_content == GTH_SIDEBAR_DIR_LIST)
-			image_delete_command_impl (NULL, window, NULL);
+			activate_action_image_delete (NULL, window);
 		else if (window->sidebar_content == GTH_SIDEBAR_CATALOG_LIST)
-			image_delete_from_catalog_command_impl (NULL, window, NULL);
+			activate_action_edit_remove_from_catalog (NULL, window);
 		break;
 
 		/* Edit comment. */
@@ -555,7 +565,7 @@ image_key_press_cb (GtkWidget   *widget,
 		if (comment_visible)
 			hide_comment_on_image ();
 
-		edit_current_edit_comment_command_impl (NULL, window, NULL);
+		activate_action_edit_edit_comment (NULL, window);
 		break;
 
 		/* Edit categories. */
@@ -570,30 +580,30 @@ image_key_press_cb (GtkWidget   *widget,
 		if (comment_visible)
 			hide_comment_on_image ();
 
-		edit_current_edit_categories_command_impl (NULL, window, NULL);
+		activate_action_edit_edit_categories (NULL, window);
 		break;
 
 		/* Flip image. */
 	case GDK_l:
 	case GDK_L:
-		alter_image_flip_command_impl (NULL, window, NULL);
+		activate_action_alter_image_flip (NULL, window);
 		break;
 
 		/* Rotate. */
 	case GDK_r:
 	case GDK_R:
 	case GDK_bracketright:
-		alter_image_rotate_90_command_impl (NULL, window, NULL);
+		activate_action_alter_image_rotate90 (NULL, window);
 		break;
 
 	case GDK_bracketleft:
-		alter_image_rotate_90_cc_command_impl (NULL, window, NULL);
+		activate_action_alter_image_rotate90cc (NULL, window);
 		break;
 
 		/* Mirror. */
 	case GDK_m:
 	case GDK_M:
-		alter_image_mirror_command_impl (NULL, window, NULL);
+		activate_action_alter_image_mirror (NULL, window);
 		break;
 
 		/* View/Hide comment */
@@ -716,7 +726,7 @@ fs_repainted_cb (GtkWidget        *widget,
 
 
 static void
-exif_fs_clicked_cb (GtkWidget *button)
+exit_fs_clicked_cb (GtkWidget *button)
 {
 	if (current_fullscreen != NULL)
 		fullscreen_stop (current_fullscreen);
@@ -757,15 +767,85 @@ image_comment_toggled_cb (GtkToggleButton  *button)
 }
 
 
+static void
+zoom_in_cb (GtkWidget  *button)
+{
+	GThumbWindow *window = current_fullscreen->related_win;
+        ImageViewer  *viewer = IMAGE_VIEWER (window->viewer);
+	if (window->image_data_visible)
+		comment_visible = TRUE;
+	image_viewer_zoom_in (viewer);
+}
+
+
+static void
+zoom_out_cb (GtkWidget  *button)
+{
+	GThumbWindow *window = current_fullscreen->related_win;
+        ImageViewer  *viewer = IMAGE_VIEWER (window->viewer);
+	if (window->image_data_visible)
+		comment_visible = TRUE;
+	image_viewer_zoom_out (viewer);
+}
+
+
+static void
+zoom_100_cb (GtkWidget  *button)
+{
+	GThumbWindow *window = current_fullscreen->related_win;
+        ImageViewer  *viewer = IMAGE_VIEWER (window->viewer);
+	if (window->image_data_visible)
+		comment_visible = TRUE;
+	image_viewer_set_zoom (viewer, 1.0);
+}
+
+
+static void
+zoom_fit_cb (GtkWidget  *button)
+{
+	GThumbWindow *window = current_fullscreen->related_win;
+        ImageViewer  *viewer = IMAGE_VIEWER (window->viewer);
+	if (window->image_data_visible)
+		comment_visible = TRUE;
+	image_viewer_zoom_to_fit (viewer);
+}
+
+
+static GtkWidget*
+create_button (const char *stock_icon,
+	       const char *label,
+	       gboolean    toggle)
+{
+	GtkWidget *button;
+	GtkWidget *image;
+	GtkWidget *button_box;
+
+	if (toggle)
+		button = gtk_toggle_button_new ();
+	else
+		button = gtk_button_new ();
+	if (pref_get_real_toolbar_style() == GTH_TOOLBAR_STYLE_TEXT_BELOW)
+		button_box = gtk_vbox_new (FALSE, 5);
+	else
+		button_box = gtk_hbox_new (FALSE, 5);
+	gtk_container_add (GTK_CONTAINER (button), button_box);
+
+	image = gtk_image_new_from_stock (stock_icon, GTK_ICON_SIZE_BUTTON);
+	gtk_box_pack_start (GTK_BOX (button_box), image, FALSE, FALSE, 0);
+
+	if (label != NULL)
+		gtk_box_pack_start (GTK_BOX (button_box), gtk_label_new (label), FALSE, FALSE, 0);
+
+	return button;
+}
+
+
 static void
 create_popup_window (void)
 {
 	GtkWidget *button;
 	GtkWidget *frame;
 	GtkWidget *hbox;
-	GtkWidget *button_hbox;
-	GtkWidget *image;
-	GdkPixbuf *icon;
 
 	popup_window = gtk_window_new (GTK_WINDOW_POPUP);
 
@@ -779,37 +859,17 @@ create_popup_window (void)
 
 	/* restore normal view */
 
-	button = gtk_button_new ();
-
-	icon = gdk_pixbuf_new_from_inline (-1, exit_fullscreen_24_rgba, FALSE, NULL);
-	image = gtk_image_new_from_pixbuf (icon);
-	g_object_unref (icon);
-
-	button_hbox = gtk_hbox_new (FALSE, 5);
-
-	gtk_box_pack_start (GTK_BOX (button_hbox), image, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (button_hbox), gtk_label_new (_("Restore Normal View")), FALSE, FALSE, 0);
-	gtk_container_add (GTK_CONTAINER (button), button_hbox);
+	button = create_button (GTHUMB_STOCK_FULLSCREEN, _("Restore Normal View"), FALSE);
 	gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
-
 	g_signal_connect (G_OBJECT (button),
 			  "clicked",
-			  G_CALLBACK (exif_fs_clicked_cb),
+			  G_CALLBACK (exit_fs_clicked_cb),
 			  NULL);
 
 	/* image info */
 
-	prop_button = button = gtk_toggle_button_new ();
-
-	image = gtk_image_new_from_stock ("gthumb-properties", GTK_ICON_SIZE_BUTTON);
-
-	button_hbox = gtk_hbox_new (FALSE, 5);
-
-	gtk_box_pack_start (GTK_BOX (button_hbox), image, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (button_hbox), gtk_label_new (_("Image Info")), FALSE, FALSE, 0);
-	gtk_container_add (GTK_CONTAINER (button), button_hbox);
+	prop_button = button = create_button (GTHUMB_STOCK_PROPERTIES, _("Image Info"), TRUE);
 	gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
-
 	g_signal_connect (G_OBJECT (button),
 			  "toggled",
 			  G_CALLBACK (image_comment_toggled_cb),
@@ -817,17 +877,8 @@ create_popup_window (void)
 
 	/* back */
 
-	button = gtk_button_new ();
-
-	image = gtk_image_new_from_stock (GTK_STOCK_GO_BACK, GTK_ICON_SIZE_BUTTON);
-
-	button_hbox = gtk_hbox_new (FALSE, 5);
-
-	gtk_box_pack_start (GTK_BOX (button_hbox), image, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (button_hbox), gtk_label_new (_("Back")), FALSE, FALSE, 0);
-	gtk_container_add (GTK_CONTAINER (button), button_hbox);
+	button = create_button (GTK_STOCK_GO_BACK, _("Back"), FALSE);
 	gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
-
 	g_signal_connect (G_OBJECT (button),
 			  "clicked",
 			  G_CALLBACK (show_prev_image_cb),
@@ -835,22 +886,49 @@ create_popup_window (void)
 
 	/* forward */
 
-	button = gtk_button_new ();
-
-	image = gtk_image_new_from_stock (GTK_STOCK_GO_FORWARD, GTK_ICON_SIZE_BUTTON);
-
-	button_hbox = gtk_hbox_new (FALSE, 5);
-
-	gtk_box_pack_start (GTK_BOX (button_hbox), image, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (button_hbox), gtk_label_new (_("Forward")), FALSE, FALSE, 0);
-	gtk_container_add (GTK_CONTAINER (button), button_hbox);
+	button = create_button (GTK_STOCK_GO_FORWARD, _("Forward"), FALSE);
 	gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
-
 	g_signal_connect (G_OBJECT (button),
 			  "clicked",
 			  G_CALLBACK (show_next_image_cb),
 			  fullscreen);
 
+	/* zoom in */
+
+	button = create_button (GTK_STOCK_ZOOM_IN, _("In"), FALSE);
+	gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+	g_signal_connect (G_OBJECT (button),
+			  "clicked",
+			  G_CALLBACK (zoom_in_cb),
+			  fullscreen);
+
+	/* zoom out */
+
+	button = create_button (GTK_STOCK_ZOOM_OUT, _("Out"), FALSE);
+	gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+	g_signal_connect (G_OBJECT (button),
+			  "clicked",
+			  G_CALLBACK (zoom_out_cb),
+			  fullscreen);
+
+	/* zoom 100 */
+
+	button = create_button (GTK_STOCK_ZOOM_100, _("1:1"), FALSE);
+	gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+	g_signal_connect (G_OBJECT (button),
+			  "clicked",
+			  G_CALLBACK (zoom_100_cb),
+			  fullscreen);
+
+	/* zoom fit */
+
+	button = create_button (GTK_STOCK_ZOOM_FIT, _("Fit"), FALSE);
+	gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+	g_signal_connect (G_OBJECT (button),
+			  "clicked",
+			  G_CALLBACK (zoom_fit_cb),
+			  fullscreen);
+
 	/**/
 
 	gtk_container_add (GTK_CONTAINER (popup_window), frame);
@@ -892,6 +970,15 @@ wmspec_change_state (gboolean   add,
 }
 
 
+static void
+close_fullscreen_window_cb (GtkWidget    *caller, 
+			    GdkEvent     *event, 
+			    FullScreen *fullscreen)
+{
+	fullscreen_stop (fullscreen);
+}
+
+
 FullScreen *
 fullscreen_new (void)
 {
@@ -902,14 +989,16 @@ fullscreen_new (void)
 	fullscreen->mouse_hide_id = 0;
 	fullscreen->window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 
+	g_signal_connect (G_OBJECT (fullscreen->window), 
+			  "delete_event",
+			  G_CALLBACK (close_fullscreen_window_cb),
+			  fullscreen);
+
 	fullscreen->wm_state_fullscreen_support = gdk_net_wm_supports (gdk_atom_intern ("_NET_WM_STATE_FULLSCREEN", FALSE));
 
 	if (! fullscreen->wm_state_fullscreen_support) {
 		gtk_widget_destroy (fullscreen->window);
 		fullscreen->window = gtk_window_new (GTK_WINDOW_POPUP);
-		gtk_window_set_default_size (GTK_WINDOW (fullscreen->window), 
-					     gdk_screen_width (), 
-					     gdk_screen_height ());
 	}
 
 	gtk_window_set_wmclass (GTK_WINDOW (fullscreen->window), "",
@@ -955,6 +1044,9 @@ void
 fullscreen_start (FullScreen   *fullscreen,
 		  GThumbWindow *window)
 {
+	int          monitor;
+	GdkRectangle rect;
+
 	g_return_if_fail (fullscreen != NULL);
 
 	if (fullscreen->related_win != NULL)
@@ -962,14 +1054,21 @@ fullscreen_start (FullScreen   *fullscreen,
 
 	current_fullscreen = fullscreen;
 
+	monitor = gdk_screen_get_monitor_at_window (gdk_screen_get_default (), GTK_WIDGET (window->app)->window);
+	gdk_screen_get_monitor_geometry (gdk_screen_get_default (), monitor, &rect);
+	gtk_window_set_default_size (GTK_WINDOW (fullscreen->window), rect.width, rect.height);
+	gtk_window_move (GTK_WINDOW (fullscreen->window), rect.x, rect.y);
+	
 	gtk_window_set_screen (GTK_WINDOW (fullscreen->window), gtk_widget_get_screen (window->app));
 	gtk_window_present (GTK_WINDOW (fullscreen->window));
 
 	window->fullscreen = TRUE;
 	fullscreen->related_win = window;
 	fullscreen->viewer = window->viewer;
+
+	/* FIXME
 	fullscreen->msg_save_modified_image = eel_gconf_get_boolean (PREF_MSG_SAVE_MODIFIED_IMAGE, TRUE);
-	eel_gconf_set_boolean (PREF_MSG_SAVE_MODIFIED_IMAGE, FALSE);
+	eel_gconf_set_boolean (PREF_MSG_SAVE_MODIFIED_IMAGE, FALSE);*/
 
 	wmspec_change_state (TRUE,
 			     fullscreen->window->window,
@@ -1067,7 +1166,9 @@ fullscreen_stop (FullScreen *fullscreen)
 	fullscreen->related_win = NULL;
 	comment_visible = FALSE;
 
+	/* FIXME
 	eel_gconf_set_boolean (PREF_MSG_SAVE_MODIFIED_IMAGE, fullscreen->msg_save_modified_image);
+	*/
 
 	/* release keyboard focus. */ 
 
@@ -1083,6 +1184,7 @@ fullscreen_stop (FullScreen *fullscreen)
 
 	gtk_widget_reparent (fullscreen->viewer, window->viewer_container);
 	gtk_widget_realize (window->viewer);
+	gtk_widget_show_all (window->viewer);
 	fullscreen->viewer = NULL;
 
 	gtk_widget_set_sensitive (window->app, TRUE);
@@ -1090,6 +1192,7 @@ fullscreen_stop (FullScreen *fullscreen)
 
 	/* restore widgets visiblity */
 
+	gtk_widget_hide (window->progress);
 	window_set_preview_content (window, window->preview_content);
 	if (! window->image_pane_visible)
 		window_hide_image_pane (window);
